@@ -51,6 +51,10 @@ utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"
 #' \item \code{B} - smoothing parameter for the seasonal component. Can either
 #' be real (if \code{seasonality="P"}) or complex (if \code{seasonality="F"})
 #' in a form b0 + ib1.
+#' \item \code{persistence} - persistence vector. This is the place, where
+#' smoothing parameters live.
+#' \item \code{transition} - transition matrix of the model.
+#' \item \code{measurement} - measurement vector of the model.
 #' \item \code{initialType} - Type of the initial values used.
 #' \item \code{initial} - the initial values of the state vector (non-seasonal).
 #' \item \code{nParam} - table with the number of estimated / provided parameters.
@@ -393,7 +397,8 @@ CreatorCES <- function(silentText=FALSE,...){
     else{
         cfType <- "MSE";
     }
-    ICValues <- ICFunction(nParam=nParam+nParamIntermittent,C=C,Etype=Etype);
+    ICValues <- ICFunction(nParam=nParam,nParamIntermittent=nParamIntermittent,
+                           C=C,Etype=Etype);
     ICs <- ICValues$ICs;
     logLik <- ICValues$llikelihood;
 
@@ -573,24 +578,25 @@ CreatorCES <- function(silentText=FALSE,...){
 
 ##### If intermittent=="a", run a loop and select the best one #####
     if(intermittent=="a"){
-        if(cfType!="MSE"){
+        if(!any(cfType==c("MSE","MAE","HAM","MSEh","MAEh","HAMh","MSCE","MACE","CHAM",
+                          "TFL","aTFL","Rounded","TSB","LogisticD","LogisticL"))){
             warning(paste0("'",cfType,"' is used as cost function instead of 'MSE'. A wrong intermittent model may be selected"),call.=FALSE);
         }
         if(!silentText){
             cat("Selecting appropriate type of intermittency... ");
         }
 # Prepare stuff for intermittency selection
-        intermittentModelsPool <- c("n","f","i","p","s");
+        intermittentModelsPool <- c("n","f","i","p","s","l");
         intermittentCFs <- intermittentICs <- rep(NA,length(intermittentModelsPool));
         intermittentModelsList <- list(NA);
-        intermittentICs[1] <- cesValues$bestIC;
+        intermittentICs[1] <- cesValues$bestIC[ic];
         intermittentCFs[1] <- cesValues$cfObjective;
 
         for(i in 2:length(intermittentModelsPool)){
             intermittentParametersSetter(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentMaker(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentModelsList[[i]] <- CreatorCES(silentText=TRUE);
-            intermittentICs[i] <- intermittentModelsList[[i]]$bestIC;
+            intermittentICs[i] <- intermittentModelsList[[i]]$bestIC[ic];
             intermittentCFs[i] <- intermittentModelsList[[i]]$cfObjective;
         }
         intermittentICs[is.nan(intermittentICs) | is.na(intermittentICs)] <- 1e+100;
@@ -858,6 +864,8 @@ CreatorCES <- function(silentText=FALSE,...){
 ##### Return values #####
     model <- list(model=modelname,timeElapsed=Sys.time()-startTime,
                   states=matvt,A=A$value,B=B$value,
+                  persistence=vecg,transition=matF,
+                  measurement=matw,
                   initialType=initialType,initial=initialValue,
                   nParam=parametersNumber,
                   fitted=y.fit,forecast=y.for,lower=y.low,upper=y.high,residuals=errors,
