@@ -1,40 +1,16 @@
-# forecast <- function(object, ...) UseMethod("forecast")
-
-
-#' Corrected Akaike's Information Criterion
-#'
-#' This function extracts AICc from "smooth" objects.
-#'
-#' AICc was proposed by Nariaki Sugiura in 1978 and is used on small samples.
-#'
-#' @aliases AICc
-#' @param object Time series model.
-#' @param ...  Some stuff.
-#' @return This function returns numeric value.
-#' @author Ivan Svetunkov, \email{ivan@@svetunkov.ru}
-#' @seealso \link[stats]{AIC}, \link[stats]{BIC}
-#' @references Kenneth P. Burnham, David R. Anderson (1998). Model Selection
-#' and Multimodel Inference. Springer Science & Business Media.
-#' @keywords htest
-#' @examples
-#'
-#' ourModel <- ces(rnorm(100,0,1),h=10)
-#'
-#' AICc(ourModel,h=10)
-#'
-#' @export AICc
-AICc <- function(object, ...) UseMethod("AICc")
-
-
 #' Functions that extract values from the fitted model
 #'
-#' These functions allow extracting orders and lags for \code{ssarima()}, \code{ges()} and \code{sma()}
-#' and type of model from \code{es()} and \code{ces()}.
+#' These functions allow extracting orders and lags for \code{ssarima()}, \code{ges()} and \code{sma()},
+#' type of model from \code{es()} and \code{ces()} and error type of all the above.
 #'
 #' \code{orders()} and \code{lags()} are useful only for SSARIMA, GES and SMA. They return \code{NA} for other functions.
 #' This can also be applied to \code{arima()}, \code{Arima()} and \code{auto.arima()} functions from stats and forecast packages.
 #' \code{modelType()} is useful only for ETS and CES. They return \code{NA} for other functions.
-#' This can also be applied to \code{ets()} function from forecast package.
+#' This can also be applied to \code{ets()} function from forecast package. Finally, \code{errorType}
+#' extracts the type of error from the model (either additive or multiplicative).
+#'
+#' @template ssAuthor
+#' @template ssKeywords
 #'
 #' @aliases orders
 #' @param object Model estimated using one of the functions of smooth package.
@@ -46,9 +22,7 @@ AICc <- function(object, ...) UseMethod("AICc")
 #' \item \code{i} - I orders.
 #' \item \code{ma} - MA orders.
 #' }
-#' @author Ivan Svetunkov, \email{ivan@@svetunkov.ru}
 #' @seealso \link[forecast]{forecast}, \link[smooth]{ssarima}
-#' @keywords ts htest
 #' @examples
 #'
 #' x <- rnorm(100,0,1)
@@ -58,11 +32,13 @@ AICc <- function(object, ...) UseMethod("AICc")
 #' orders(ourModel)
 #' lags(ourModel)
 #' modelType(ourModel)
+#' errorType(ourModel)
 #' # And as another example it does the opposite for ges() and ssarima()
 #' ourModel <- ges(x, h=10, orders=c(1,1), lags=c(1,4))
 #' orders(ourModel)
 #' lags(ourModel)
 #' modelType(ourModel)
+#' errorType(ourModel)
 #'
 #' # Finally these values can be used for simulate functions or original functions.
 #' ourModel <- auto.ssarima(x)
@@ -81,9 +57,226 @@ lags <- function(object, ...) UseMethod("lags")
 #' @aliases modelType
 #' @rdname orders
 #' @export modelType
-modelType <-  function(object, ...) UseMethod("modelType")
+modelType <- function(object, ...) UseMethod("modelType")
+
+modelLags <- function(object, ...) UseMethod("modelLags")
+
+#' @aliases errorType
+#' @rdname orders
+#' @export errorType
+errorType <- function(object, ...) UseMethod("errorType")
 
 ##### Likelihood function and stuff #####
+
+#' @importFrom greybox AICc
+#' @export
+AICc.smooth <- function(object, ...){
+    llikelihood <- logLik(object);
+    nParamAll <- attributes(llikelihood)$df;
+    llikelihood <- llikelihood[1:length(llikelihood)];
+
+    if(!is.null(object$imodel)){
+        obs <- sum(object$fitted!=0);
+        nParamSizes <- nParamAll - object$nParam[1,3];
+
+        IC <- (2*nParamAll - 2*llikelihood +
+                   2*nParamSizes*(nParamSizes + 1) / (obs - nParamSizes - 1));
+    }
+    else{
+        obs <- nobs(object);
+
+        IC <- 2*nParamAll - 2*llikelihood + 2 * nParamAll * (nParamAll + 1) / (obs - nParamAll - 1);
+    }
+
+    return(IC);
+}
+
+#' Function returns the covariance matrix of conditional multiple steps ahead forecast errors
+#'
+#' This function extracts covariance matrix of 1 to h steps ahead forecast errors for
+#' \code{ssarima()}, \code{ges()}, \code{sma()}, \code{es()} and \code{ces()} models.
+#'
+#' The function returns either scalar (if it is a non-smooth model)
+#' or the matrix of (h x h) size with variances and covariances of 1 to h steps ahead
+#' forecat errors. This is currently done based on empirical values. The analytical ones
+#' are more complicated.
+#'
+#' @template ssAuthor
+#' @template ssKeywords
+#'
+#' @param object Model estimated using one of the functions of smooth package.
+#' @param type What method to use in order to produce covariance matrix:
+#' \enumerate{
+#' \item \code{analytical} - based on the state-space structure of the model and the
+#' one-step-ahead forecast error. This works for pure additive and pure multiplicative
+#' models. The values for the mixed models might be off.
+#' \item \code{empirical} - based on the in-sample 1 to h steps ahead forecast errors
+#' (works fine on larger samples);
+#' \item \code{simulated} - the data is simulated from the estimated model, then the
+#' same model is applied to it and then the empirical 1 to h steps ahead forecast
+#' errors are produced;
+#' }
+#' @param ... Other parameters passed to simulate function (if \code{type="simulated"}
+#' is used). These are \code{obs}, \code{nsim} and \code{seed}. By default
+#' \code{obs=1000}, \code{nsim=100}. This approach increases the accuracy of
+#' covariance matrix on small samples and intermittent data;
+#' @return Scalar in cases of non-smooth functions. (h x h) matrix otherwise.
+#'
+#' @seealso \link[smooth]{orders}
+#' @examples
+#'
+#' x <- rnorm(100,0,1)
+#'
+#' # A simple example with a 5x5 covariance matrix
+#' ourModel <- ces(x, h=5)
+#' covar(ourModel)
+#'
+#' @rdname covar
+#' @export covar
+covar <-  function(object, type=c("analytical","empirical","simulated"), ...) UseMethod("covar")
+
+#' @export
+covar.default <- function(object, type=c("analytical","empirical","simulated"), ...){
+    # Function extracts the conditional variances from the model
+    return(sigma(object)^2);
+}
+
+#' @aliases covar.smooth
+#' @rdname covar
+#' @export
+covar.smooth <- function(object, type=c("analytical","empirical","simulated"), ...){
+    # Function extracts the conditional variances from the model
+    type <- substr(type[1],1,1);
+
+    if(is.null(object$persistence) & any(type==c("a","s"))){
+        warning(paste0("The provided model does not contain the components necessary for the ",
+                       "derivation of the covariance matrix.\n",
+                       "Did you combine forecasts? Switching to 'empirical'"),
+                call.=FALSE);
+        type <- "e";
+    }
+
+    if(!is.null(object$imodel) & type=="e"){
+        warning(paste0("Empirical covariance matrix can be very inaccurate in cases of ",
+                       "intemittent models.\nWe recommend using type='s' or type='a' instead."),
+                call.=FALSE);
+    }
+
+    # Empirical covariance matrix
+    if(type=="e"){
+        if(errorType(object)=="A"){
+            errors <- object$errors;
+        }
+        else{
+            errors <- log(1 + object$errors);
+        }
+        if(!is.null(object$imodel)){
+            obs <- t((errors!=0)*1) %*% (errors!=0)*1;
+            obs[obs==0] <- 1;
+            df <- obs - nParam(object);
+            df[df<=0] <- obs[df<=0];
+        }
+        else{
+            obs <- matrix(nobs(object),ncol(errors),ncol(errors));
+            df <- obs - nParam(object);
+            df[df<=0] <- obs[df<=0];
+        }
+        covarMat <- t(errors) %*% errors / df;
+    }
+    # Simulated covariance matrix
+    else if(type=="s"){
+        ellipsis <- list(...);
+        if(any(names(ellipsis)=="obs")){
+            obs <- ellipsis$obs;
+        }
+        else{
+            obs <- length(getResponse(object));
+        }
+        if(any(names(ellipsis)=="nsim")){
+            nsim <- ellipsis$nsim;
+        }
+        else{
+            nsim <- 1000;
+        }
+        if(any(names(ellipsis)=="seed")){
+            seed <- ellipsis$seed;
+        }
+        else{
+            seed <- NULL;
+        }
+
+        h <- length(object$forecast);
+        if(gregexpr("ETS",object$model)!=-1){
+            smoothFunction <- es;
+        }
+        # GES models
+        else if(gregexpr("GES",object$model)!=-1){
+            smoothFunction <- ges;
+        }
+        # SSARIMA models
+        else if(gregexpr("ARIMA",object$model)!=-1){
+            smoothFunction <- ssarima;
+        }
+        # CES models
+        else if(gregexpr("CES",object$model)!=-1){
+            smoothFunction <- ces;
+        }
+        # SMA models
+        else if(gregexpr("SMA",object$model)!=-1){
+            smoothFunction <- sma;
+        }
+
+        covarArray <- array(NA,c(h,h,nsim));
+
+        newData <- simulate(object, nsim=nsim, obs=obs, seed=seed);
+        for(i in 1:nsim){
+            # Apply the model to the simulated data
+            smoothModel <- smoothFunction(newData$data[,i], model=object, h=h);
+            # Remove first h-1 and last values.
+            errors <- smoothModel$errors[h-1+1:obs,];
+
+            # Transform errors if needed
+            if(errorType(object)=="M"){
+                # Remove zeroes if they are present
+                errors <- errors[!apply(errors==0,1,any),];
+                errors <- log(1 + errors);
+            }
+
+            # Calculate covariance matrix
+            if(!is.null(object$imodel)){
+                obsInSample <- t((errors!=0)*1) %*% (errors!=0)*1;
+                obsInSample[obsInSample==0] <- 1;
+            }
+            else{
+                obsInSample <- matrix(nrow(errors),ncol(errors),ncol(errors));
+            }
+            covarArray[,,i] <- t(errors) %*% errors / obsInSample;
+        }
+
+        covarMat <- apply(covarArray,c(1,2),mean);
+    }
+    # Analytical covariance matrix
+    else if(type=="a"){
+        if(!is.null(object$imodel)){
+            ot <- (residuals(object)!=0)*1;
+        }
+        else{
+            ot <- rep(1,length(residuals(object)));
+        }
+        h <- length(object$forecast);
+        lagsModel <- modelLags(object);
+        s2 <- sigma(object)^2;
+        persistence <- matrix(object$persistence,length(object$persistence),1);
+        transition <- object$transition;
+        measurement <- object$measurement;
+
+        covarMat <- covarAnal(lagsModel, h, nComponents, measurement, transition, persistence, s2);
+
+    }
+    return(covarMat);
+    # correlation matrix: covar(test) / sqrt(diag(covar(test)) %*% t(diag(covar(test))))
+}
+
 #' @importFrom stats logLik
 #' @export
 logLik.smooth <- function(object,...){
@@ -133,35 +326,7 @@ nobs.iss <- function(object, ...){
     return(length(object$fitted));
 }
 
-#' Number of parameters in the model
-#'
-#' This function returns the number of estimated parameters in the model
-#'
-#' This is a very basic and a simple function which does what it says:
-#' extracts number of parameters in the estimated model.
-#'
-#' @aliases nParam
-#' @param object Time series model.
-#' @param ... Some other parameters passed to the method.
-#' @return This function returns a numeric value.
-#' @author Ivan Svetunkov, \email{ivan@@svetunkov.ru}
-#' @seealso \link[stats]{nobs}, \link[stats]{logLik}
-#' @keywords htest
-#' @examples
-#'
-#' ourModel <- ces(rnorm(100,0,1),h=10)
-#'
-#' nParam(ourModel)
-#'
-#' @importFrom stats coefficients
-#' @export nParam
-nParam <- function(object, ...) UseMethod("nParam")
-
-#' @export
-nParam.default <- function(object, ...){
-    # The length of the vector of parameters + variance
-    return(length(coefficients(object))+1);
-}
+#' @importFrom greybox nParam
 
 #' @export
 nParam.smooth <- function(object, ...){
@@ -180,6 +345,179 @@ nParam.iss <- function(object, ...){
     return(object$nParam);
 }
 
+#' Prediction Likelihood Score
+#'
+#' Function estimates Prediction Likelihood Score for the provided model
+#'
+#' Prediction likelihood score (PLS) is based on either normal or log-normal
+#' distribution of errors. This is extracted from the provided model. The likelihood
+#' based onthe distribution of 1 to h steps ahead forecast errors is used in the process.
+#'
+#' @template ssAuthor
+#' @template ssKeywords
+#'
+#' @param object The model estimated using smooth functions. This thing also accepts
+#' other models (e.g. estimated using functions from forecast package), but may not always
+#' work properly with them.
+#' @param holdout The values for the holdout part of the sample. If the model was fitted
+#' on the data with the \code{holdout=TRUE}, then the parameter is not needed.
+#' @param ... Parameters passed to covar function. The function is called in order to get
+#' the covariance matrix of 1 to h steps ahead forecast errors.
+#'
+#' @return A value of the log-likelihood.
+#' @references \itemize{
+#' %\item Eltoft, T., Taesu, K., Te-Won, L. (2006). On the multivariate Laplace
+#' distribution. IEEE Signal Processing Letters. 13 (5): 300-303.
+#' \doi{10.1109/LSP.2006.870353} - this is not yet used in the function.
+#' \item Snyder, R. D., Ord, J. K., Beaumont, A., 2012. Forecasting the intermittent
+#' demand for slow-moving inventories: A modelling approach. International
+#' Journal of Forecasting 28 (2), 485-496.
+#' \item Kolassa, S., 2016. Evaluating predictive count data distributions in retail
+#' sales forecasting. International Journal of Forecasting 32 (3), 788-803..
+#' }
+#' @examples
+#'
+#' # Generate data, apply es() with the holdout parameter and calculate PLS
+#' x <- rnorm(100,0,1)
+#' ourModel <- es(x, h=10, holdout=TRUE, intervals=TRUE)
+#' pls(ourModel, type="a")
+#' pls(ourModel, type="e")
+#' pls(ourModel, type="s", obs=100, nsim=100)
+#'
+#' @rdname pls
+#' @export pls
+pls <-  function(object, holdout=NULL, ...) UseMethod("pls")
+# Function calculates PLS based on the provided model
+
+#' @importFrom stats dnorm
+#' @export
+pls.default <- function(object, holdout=NULL, ...){
+    if(is.null(holdout)){
+        stop("We need the values from the holdout in order to proceed.",
+             call.=FALSE);
+    }
+    h <- length(holdout);
+    yForecast <- forecast(object, h=h)$mean;
+
+    return(sum(dnorm(holdout,yForecast,sigma(object),log=TRUE)));
+}
+
+#' @rdname pls
+#' @aliases pls.smooth
+#' @export
+pls.smooth <- function(object, holdout=NULL, ...){
+    # If holdout is provided, check it and use it. Otherwise try extracting from the model
+    yForecast <- object$forecast;
+    covarMat <- covar(object, ...);
+    if(!is.null(holdout)){
+        if(length(yForecast)!=length(holdout)){
+            if(is.null(object$holdout)){
+                stop("The forecast of the model does not correspond to the provided holdout.",
+                     call.=FALSE);
+            }
+            else{
+                holdout <- object$holdout;
+            }
+        }
+    }
+    else{
+        if(all(is.na(object$holdout))){
+            stop("No values for the holdout are available. Cannot proceed.",
+                 call.=FALSE);
+        }
+        holdout <- object$holdout;
+    }
+    h <- length(holdout);
+
+    Etype <- errorType(object);
+    cfType <- object$cfType;
+    if(any(cfType==c("MAE","MAEh","TMAE","GTMAE","MACE"))){
+        cfType <- "MAE";
+    }
+    else if(any(cfType==c("HAM","HAMh","THAM","GTHAM","CHAM"))){
+        cfType <- "HAM";
+    }
+    else{
+        cfType <- "MSE";
+    }
+
+    densityFunction <- function(cfType, ...){
+        if(cfType=="MAE"){
+        # This is a simplification. The real multivariate Laplace is bizarre!
+            b <- sqrt(diag(covarMat)/2);
+            plsValue <- sum(dlaplace(errors, 0, b, log=TRUE));
+        }
+        else if(cfType=="HAM"){
+        # This is a simplification. We don't have multivariate HAM yet.
+            b <- (diag(covarMat)/120)^0.25;
+            plsValue <- sum(ds(errors, 0, b, log=TRUE));
+        }
+        else{
+        # Here and later in the code the abs() is needed for weird cases of wrong covarMat
+            plsValue <- -as.vector(log(2*pi*abs(det(covarMat)))/2 +
+                                       (t(errors) %*% solve(covarMat) %*% errors) / 2);
+        }
+        return(plsValue);
+    }
+
+    # Additive models
+    if(Etype=="A"){
+        # Non-intermittent data
+        if(is.null(object$imodel)){
+            errors <- holdout - yForecast;
+            plsValue <- densityFunction(cfType, errors, covarMat);
+        }
+        # Intermittent data
+        else{
+            ot <- holdout!=0;
+            pForecast <- object$imodel$forecast;
+            errors <- holdout - yForecast / pForecast;
+            if(all(ot)){
+                plsValue <- densityFunction(cfType, errors, covarMat) + sum(log(pForecast));
+            }
+            else if(all(!ot)){
+                plsValue <- sum(log(1-pForecast));
+            }
+            else{
+                errors[!ot] <- 0;
+
+                plsValue <- densityFunction(cfType, errors, covarMat);
+                plsValue <- plsValue + sum(log(pForecast[ot])) + sum(log(1-pForecast[!ot]));
+            }
+        }
+    }
+    # Multiplicative models
+    else{
+        # Non-intermittent data
+        if(is.null(object$imodel)){
+            errors <- log(holdout) - log(yForecast);
+            plsValue <- densityFunction(cfType, errors, covarMat) - sum(log(holdout));
+        }
+        # Intermittent data
+        else{
+            ot <- holdout!=0;
+            pForecast <- object$imodel$forecast;
+            errors <- log(holdout) - log(yForecast / pForecast);
+            if(all(ot)){
+                plsValue <- (densityFunction(cfType, errors, covarMat) - sum(log(holdout)) +
+                             sum(log(pForecast)));
+            }
+            else if(all(!ot)){
+                plsValue <- sum(log(1-pForecast));
+            }
+            else{
+                errors[!ot] <- 0;
+
+                plsValue <- densityFunction(cfType, errors, covarMat) - sum(log(holdout[ot]));
+                plsValue <- plsValue + sum(log(pForecast[ot])) + sum(log(1-pForecast[!ot]));
+            }
+        }
+    }
+
+    return(plsValue);
+}
+
+
 #' Point likelihood values
 #'
 #' This function returns a vector of logarithms of likelihoods for each observation
@@ -226,25 +564,16 @@ pointLik.default <- function(object, ...){
 
 #' @export
 pointLik.smooth <- function(object, ...){
-    if(!any(class(object)=="smooth")){
-        stop("Sorry, but we do not support this class yet.",call.=FALSE);
-    }
-
     obs <- nobs(object);
     errors <- residuals(object);
     s2 <- sigma(object)^2;
     likValues <- vector("numeric",obs);
 
-    if(gregexpr("ETS",object$model)!=-1){
-        if(substr(modelType(object),1,1)=="A"){
-            likValues <- -1/2 * log(2*pi*s2) - 1/2 * errors^2 / s2;
-        }
-        else{
-            likValues <- -1/2 * log(2*pi*s2) - 1/2 * errors^2 / s2 - log(getResponse(object));
-        }
+    if(errorType(object)=="A"){
+        likValues <- -1/2 * log(2*pi*s2) - 1/2 * errors^2 / s2;
     }
     else{
-        likValues <- -1/2 * log(2*pi*s2) - 1/2 * errors^2 / s2;
+        likValues <- -1/2 * log(2*pi*s2) - 1/2 * errors^2 / s2 - log(getResponse(object));
     }
     return(likValues);
 }
@@ -255,20 +584,6 @@ sigma.smooth <- function(object, ...){
     return(sqrt(object$s2));
 }
 
-##### IC functions #####
-#' @export
-AICc.default <- function(object, ...){
-    obs <- nobs(object);
-
-    llikelihood <- logLik(object);
-    nParam <- attributes(llikelihood)$df;
-    llikelihood <- llikelihood[1:length(llikelihood)];
-
-    IC <- 2*nParam - 2*llikelihood + 2 * nParam * (nParam + 1) / (obs - nParam - 1);
-
-    return(IC);
-}
-
 #### Extraction of parameters of models ####
 #' @export
 coef.smooth <- function(object, ...)
@@ -437,6 +752,21 @@ getResponse.smooth.forecast <- function(object, ...){
 #### Function extracts lags of provided model ####
 #' @export
 lags.default <- function(object, ...){
+    lags <- NA;
+    return(lags);
+}
+
+#' @export
+lags.Arima <- function(object, ...){
+    model <- object$arma;
+
+    lags <- c(1,model[5]);
+
+    return(lags);
+}
+
+#' @export
+lags.smooth <- function(object, ...){
     model <- object$model;
     if(!is.null(model)){
         if(gregexpr("GES",model)!=-1){
@@ -458,6 +788,33 @@ lags.default <- function(object, ...){
         else if(gregexpr("SMA",model)!=-1){
             lags <- 1;
         }
+        else if(gregexpr("ETS",model)!=-1){
+            modelName <- modelType(object);
+            lags <- c(1);
+            if(substr(modelName,nchar(modelName),nchar(modelName))!="N"){
+                lags <- c(lags,frequency(getResponse(object)));
+            }
+        }
+        else if(gregexpr("CES",model)!=-1){
+            modelName <- modelType(object);
+            dataFreq <- frequency(getResponse(object));
+            if(modelName=="none"){
+                lags <- c(1);
+            }
+            else if(modelName=="simple"){
+                lags <- c(dataFreq);
+            }
+            else if(modelName=="partial"){
+                lags <- c(1,dataFreq);
+            }
+            else if(modelName=="full"){
+                lags <- c(1,dataFreq);
+            }
+            else{
+                stop("Sorry, but we cannot identify the type of the provided model.",
+                     call.=FALSE);
+            }
+        }
         else{
             lags <- NA;
         }
@@ -470,54 +827,138 @@ lags.default <- function(object, ...){
 }
 
 #' @export
-lags.Arima <- function(object, ...){
-    model <- object$arma;
+lags.smooth.sim <- lags.smooth;
 
-    lags <- c(1,model[5]);
+#### Function extracts type of error in the model: "A" or "M" ####
+#' @export
+errorType.default <- function(object, ...){
+    return("A");
+}
 
-    return(lags);
+#' @export
+errorType.smooth <- function(object, ...){
+    # ETS models
+    if(gregexpr("ETS",object$model)!=-1){
+        if(any(substr(modelType(object),1,1)==c("A","X"))){
+            Etype <- "A";
+        }
+        else if(any(substr(modelType(object),1,1)==c("M","Y"))){
+            Etype <- "M";
+        }
+        else{
+            stop("Sorry, but we cannot calculate PLS for this type of model",
+                 call.=FALSE);
+        }
+    }
+    # GES models
+    else if(gregexpr("GES",object$model)!=-1){
+        if(gregexpr("MGES",object$model)!=-1){
+            Etype <- "M";
+        }
+        else{
+            Etype <- "A";
+        }
+    }
+    # SSARIMA models
+    else if(gregexpr("ARIMA",object$model)!=-1){
+        Etype <- "A";
+    }
+    # CES models
+    else if(gregexpr("CES",object$model)!=-1){
+        Etype <- "A";
+    }
+    # SMA models
+    else if(gregexpr("SMA",object$model)!=-1){
+        Etype <- "A";
+    }
+    else{
+        stop(paste0("Sorry but we cannot identify error type for the model '",object$model),
+             call.=FALSE);
+    }
+    return(Etype);
+}
+
+##### Function returns the modellags from the model - internal function #####
+modelLags.default <- function(object, ...){
+    modelLags <- NA;
+    if(gregexpr("ETS",object$model)!=-1){
+        modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+    }
+    else if(gregexpr("GES",object$model)!=-1){
+        modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+    }
+    else if(gregexpr("ARIMA",object$model)!=-1){
+        ordersARIMA <- orders(object);
+        nComponents <- max(ordersARIMA$ar %*% lags(object) + ordersARIMA$i %*% lags(object),
+                           ordersARIMA$ma %*% lags(object));
+        modelLags <- matrix(rep(1,times=nComponents),ncol=1);
+        if(is.numeric(object$constant)){
+            modelLags <- rbind(modellags,1);
+        }
+    }
+    else if(gregexpr("CES",object$model)!=-1){
+        modelLags <- matrix(rep(lags(object),times=orders(object)),ncol=1);
+    }
+    else if(gregexpr("SMA",object$model)!=-1){
+        modelLags <- matrix(rep(1,times=orders(object)),ncol=1);
+    }
+    return(modelLags);
 }
 
 #### Function extracts type of model. For example "AAN" from ets ####
 #' @export
 modelType.default <- function(object, ...){
-    model <- object$model;
-    if(!is.null(model)){
-        if(gregexpr("ETS",model)!=-1){
-            modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
-        }
-        else if(gregexpr("CES",model)!=-1){
-            modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
-            if(modelType=="n"){
-                modelType <- "none";
-            }
-            else if(modelType=="s"){
-                modelType <- "simple";
-            }
-            else if(modelType=="p"){
-                modelType <- "partial";
-            }
-            else{
-                modelType <- "full";
-            }
-        }
-        else{
-            modelType <- NA;
-        }
-    }
-    else{
+    modelType <- NA;
+    if(is.null(object$model)){
         if(any(gregexpr("ets",object$call)!=-1)){
             model <- object$method;
             modelType <- gsub(",","",substring(model,5,nchar(model)-1));
         }
     }
+    return(modelType);
+}
+
+#' @export
+modelType.smooth <- function(object, ...){
+    model <- object$model;
+
+    if(gregexpr("ETS",model)!=-1){
+        modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
+    }
+    else if(gregexpr("CES",model)!=-1){
+        modelType <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
+        if(modelType=="n"){
+            modelType <- "none";
+        }
+        else if(modelType=="s"){
+            modelType <- "simple";
+        }
+        else if(modelType=="p"){
+            modelType <- "partial";
+        }
+        else{
+            modelType <- "full";
+        }
+    }
+    else{
+        modelType <- NA;
+    }
 
     return(modelType);
 }
 
+#' @export
+modelType.smooth.sim <- modelType.smooth;
+
 #### Function extracts orders of provided model ####
 #' @export
 orders.default <- function(object, ...){
+    orders <- NA;
+    return(orders);
+}
+
+#' @export
+orders.smooth <- function(object, ...){
     model <- object$model;
     if(!is.null(model)){
         if(gregexpr("GES",model)!=-1){
@@ -542,6 +983,35 @@ orders.default <- function(object, ...){
         else if(gregexpr("SMA",model)!=-1){
             orders <- as.numeric(substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1));
         }
+        else if(gregexpr("ETS",model)!=-1){
+            modelName <- modelType(object);
+            orders <- 1;
+            if(substr(modelName,2,2)!="N"){
+                orders <- 2;
+            }
+            if(substr(modelName,nchar(modelName),nchar(modelName))!="N"){
+                orders <- c(orders, 1);
+            }
+        }
+        else if(gregexpr("CES",model)!=-1){
+            modelName <- modelType(object);
+            if(modelName=="none"){
+                orders <- 2;
+            }
+            else if(modelName=="simple"){
+                orders <- 2;
+            }
+            else if(modelName=="partial"){
+                orders <- c(2,1);
+            }
+            else if(modelName=="full"){
+                orders <- c(2,2);
+            }
+            else{
+                stop("Sorry, but we cannot identify the type of the provided model.",
+                     call.=FALSE);
+            }
+        }
         else{
             orders <- NA;
         }
@@ -553,6 +1023,9 @@ orders.default <- function(object, ...){
     return(orders);
 }
 
+#' @export
+orders.smooth.sim <- orders.smooth;
+
 #' @export
 orders.Arima <- function(object, ...){
     model <- object$arma;
@@ -894,6 +1367,9 @@ print.iss <- function(x, ...){
     else if(intermittent=="l"){
         intermittent <- "Logistic probability";
     }
+    else if(intermittent=="s"){
+        intermittent <- "Interval-based with SBA correction";
+    }
     else{
         intermittent <- "None";
     }
@@ -927,8 +1403,7 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
     }
 
     if(gregexpr("ETS",object$model)!=-1){
-        model <- object$model;
-        model <- substring(model,unlist(gregexpr("\\(",model))+1,unlist(gregexpr("\\)",model))-1);
+        model <- modelType(object);
         if(any(unlist(gregexpr("C",model))==-1)){
             if(substr(model,1,1)=="A"){
                 randomizer <- "rnorm";
@@ -962,7 +1437,7 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
         }
     }
     else if(gregexpr("CES",object$model)!=-1){
-        model <- substring(object$model,unlist(gregexpr("\\(",object$model))+1,unlist(gregexpr("\\)",object$model))-1);
+        model <- modelType(object);
         initial <- object$initial;
         randomizer <- "rnorm";
         simulatedData <- sim.ces(seasonality=model,
@@ -972,14 +1447,14 @@ simulate.smooth <- function(object, nsim=1, seed=NULL, obs=NULL, ...){
     }
     else if(gregexpr("GES",object$model)!=-1){
         model <- object$model;
-        orders <- as.numeric(substring(model,unlist(gregexpr("\\[",model))-1,unlist(gregexpr("\\[",model))-1));
-        lags <- as.numeric(substring(model,unlist(gregexpr("\\[",model))+1,unlist(gregexpr("\\]",model))-1));
+        orders <- orders(object);
+        lags <- lags(object);
         initial <- object$initial;
         randomizer <- "rnorm";
         simulatedData <- sim.ges(orders=orders, lags=lags, frequency=frequency(object$actuals), measurement=object$measurement,
                                  transition=object$transition, persistence=object$persistence, initial=object$initial,
                                  obs=obs, nsim=nsim,
-                                 iprob=object$iprob[length(object$iprob)], randomizer=randomizer, mean=0, sd=sqrt(object$s2),...);
+                                 iprob=object$iprob[length(object$iprob)], randomizer=randomizer, mean=0, sd=sigma(object),...);
 
     }
     else if(gregexpr("SMA",object$model)!=-1){
