@@ -90,6 +90,7 @@ utils::globalVariables(c("normalizer","constantValue","constantRequired","consta
 #' \item \code{transition} - matrix F.
 #' \item \code{persistence} - the persistence vector. This is the place, where
 #' smoothing parameters live.
+#' \item \code{measurement} - measurement vector of the model.
 #' \item \code{AR} - the matrix of coefficients of AR terms.
 #' \item \code{I} - the matrix of coefficients of I terms.
 #' \item \code{MA} - the matrix of coefficients of MA terms.
@@ -445,7 +446,8 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
         cfType <- "MSE";
     }
 
-    ICValues <- ICFunction(nParam=nParam+nParamIntermittent,C=C,Etype=Etype);
+    ICValues <- ICFunction(nParam=nParam,nParamIntermittent=nParamIntermittent,
+                           C=C,Etype=Etype);
     ICs <- ICValues$ICs;
     bestIC <- ICs["AICc"];
     logLik <- ICValues$llikelihood;
@@ -614,24 +616,25 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
 
 ##### If intermittent=="a", run a loop and select the best one #####
     if(intermittent=="a"){
-        if(cfType!="MSE"){
+        if(!any(cfType==c("MSE","MAE","HAM","MSEh","MAEh","HAMh","MSCE","MACE","CHAM",
+                          "TFL","aTFL","Rounded","TSB","LogisticD","LogisticL"))){
             warning(paste0("'",cfType,"' is used as cost function instead of 'MSE'. A wrong intermittent model may be selected"),call.=FALSE);
         }
         if(!silentText){
             cat("Selecting appropriate type of intermittency... ");
         }
 # Prepare stuff for intermittency selection
-        intermittentModelsPool <- c("n","f","c","t","s");
+        intermittentModelsPool <- c("n","f","i","p","s","l");
         intermittentCFs <- intermittentICs <- rep(NA,length(intermittentModelsPool));
         intermittentModelsList <- list(NA);
-        intermittentICs[1] <- ssarimaValues$bestIC;
+        intermittentICs[1] <- ssarimaValues$bestIC[ic];
         intermittentCFs[1] <- ssarimaValues$cfObjective;
 
         for(i in 2:length(intermittentModelsPool)){
             intermittentParametersSetter(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentMaker(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentModelsList[[i]] <- CreatorSSARIMA(silentText=TRUE);
-            intermittentICs[i] <- intermittentModelsList[[i]]$bestIC;
+            intermittentICs[i] <- intermittentModelsList[[i]]$bestIC[ic];
             intermittentCFs[i] <- intermittentModelsList[[i]]$cfObjective;
             # if(intermittentICs[i]>intermittentICs[i-1]){
             #     break;
@@ -982,6 +985,7 @@ CreatorSSARIMA <- function(silentText=FALSE,...){
 ##### Return values #####
     model <- list(model=modelname,timeElapsed=Sys.time()-startTime,
                   states=matvt,transition=matF,persistence=vecg,
+                  measurement=matw,
                   AR=ARterms,I=Iterms,MA=MAterms,constant=const,
                   initialType=initialType,initial=initialValue,
                   nParam=parametersNumber,
