@@ -12,8 +12,8 @@ ges(M3$N2457$x, h=18, holdout=TRUE)
 ## ----ges_N2457_2[1]_1[12]------------------------------------------------
 ges(M3$N2457$x, h=18, holdout=TRUE, orders=c(2,1), lags=c(1,12))
 
-## ----ges_N2457_1[1]------------------------------------------------------
-ges(M3$N2457$x, h=18, holdout=TRUE, orders=c(1), lags=c(1), intervals=TRUE)
+## ----Autoges_N2457_1[1]--------------------------------------------------
+auto.ges(M3$N2457$x, h=18, holdout=TRUE, intervals=TRUE, silent=FALSE)
 
 ## ----ges_N2457_predefined------------------------------------------------
 	transition <- matrix(c(1,0,0,1,1,0,0,0,1),3,3)
