@@ -30,7 +30,7 @@ test_that("Test ETS(MXM) with AIC on N2568$x", {
 # Test trace cost function for ETS
 testModel <- es(Mcomp::M3$N2568$x, model="MAdM", h=18, holdout=TRUE, silent=TRUE, intervals=TRUE)
 test_that("Test AIC of ETS on N2568$x", {
-    expect_equal(as.numeric(round(AIC(testModel),2)), as.numeric(round(testModel$ICs["AIC"],2)));
+    expect_equal(as.numeric(round(AIC(testModel),2)), as.numeric(round(testModel$ICs[1,"AIC"],2)));
 })
 
 # Test how different passed values are accepted by ETS
@@ -46,7 +46,7 @@ x <- cbind(c(rep(0,25),1,rep(0,43)),c(rep(0,10),1,rep(0,58)));
 y <- ts(c(Mcomp::M3$N1457$x,Mcomp::M3$N1457$xx),frequency=12);
 testModel <- es(y, h=18, holdout=TRUE, xreg=x, updateX=TRUE, silent=TRUE, intervals="np")
 test_that("Check exogenous variables for ETS on N1457", {
-    expect_equal(suppressWarnings(es(y, "ANN", h=18, holdout=TRUE, xreg=x, cfType="aTMSE", silent=TRUE)$model), testModel$model);
+    expect_equal(suppressWarnings(es(y, "MNN", h=18, holdout=TRUE, xreg=x, cfType="aTMSE", silent=TRUE)$model), testModel$model);
     expect_equal(suppressWarnings(forecast(testModel, h=18, holdout=FALSE)$method), testModel$model);
 })
 
